{"cells": [{"cell_type": "markdown", "id": "0b15b497", "metadata": {}, "source": ["# Influence of correlation on geometry: HNO$_3$ molecule\n", "\n", "The influence of electron correlation on structural parameters such as\n", "bond lengths and angles is not negligible. Hartree-Fock calculations may\n", "give bond lengths that deviate considerably from the experimental value.\n", "Comparably cheap methods such as MPn may often remedy sufficiently for\n", "this problem, but some smaller deviations may remain. The most rigorous\n", "approach in such cases would be *e.g.* a full configuration interaction\n", "(**full CI**/FCI), but this is computationally untractable for any larger\n", "molecule using a reasonable basis. One may therefore restrict the method\n", "to singly and doubly excited Slater determinants, assuming that the main\n", "effects of correlation are captured well enough in such a truncation.\n", "However, such truncations introduce size-consistency problems."]}, {"cell_type": "markdown", "id": "cb2ad91b-6b5b-4086-af86-dae18c128879", "metadata": {}, "source": ["In this exercise, we will assess and comment on the performance of HF, MP2 and MP3 with respect to the prediction of bond lengths and bond angles in HNO$_3$ in reference to the experimental values ([*Phys. Chem. Chem. Phys.*, ***2014*** 16, *19437*](https://pubs.rsc.org/en/content/articlehtml/2014/cp/c4cp02792b))."]}, {"cell_type": "markdown", "id": "01a12e6f", "metadata": {}, "source": [" ![image](../../images/HNO3.png)"]}, {"cell_type": "markdown", "id": "bf20f5df", "metadata": {}, "source": [":::{admonition} Exercise 8\n", ":class: exercise \n", "\n", "Assess and comment on the performance of the different methods with respect to experimental values. Can you think of possible reasons for differences between the experimental and computed values?\n", ":::"]}, {"cell_type": "code", "execution_count": 50, "id": "03cea67c", "metadata": {}, "outputs": [], "source": ["import psi4\n", "import py3Dmol\n", "import pandas as pd\n", "import numpy as np\n", "\n", "import sys\n", "sys.path.append(\"..\")\n", "from helpers import *\n", "\n", "psi4.core.clean_options()\n"]}, {"cell_type": "code", "execution_count": 51, "id": "7c21bbc9", "metadata": {}, "outputs": [], "source": ["hno3 = psi4.geometry(\"\"\"\n", "0 1\n", "symmetry c1\n", "O           1.326  -0.373  -0.127\n", "N           0.015  -0.015   0.415\n", "O          -0.329   1.292  -0.145\n", "O          -0.944  -0.973  -0.135\n", "H          -1.811  -0.738   0.223\"\"\")"]}, {"cell_type": "code", "execution_count": 52, "id": "a587f1aa", "metadata": {}, "outputs": [{"data": {"application/3dmoljs_load.v0": "<div id=\"3dmolviewer_16692193574285905\"  style=\"position: relative; width: 400px; height: 400px\">\n        <p id=\"3dmolwarning_16692193574285905\" style=\"background-color:#ffcccc;color:black\">You appear to be running in JupyterLab (or JavaScript failed to load for some other reason).  You need to install the 3dmol extension: <br>\n        <tt>jupyter labextension install jupyterlab_3dmol</tt></p>\n        </div>\n<script>\n\nvar loadScriptAsync = function(uri){\n  return new Promise((resolve, reject) => {\n    var tag = document.createElement('script');\n    tag.src = uri;\n    tag.async = true;\n    tag.onload = () => {\n      resolve();\n    };\n  var firstScriptTag = document.getElementsByTagName('script')[0];\n  firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);\n});\n};\n\nif(typeof $3Dmolpromise === 'undefined') {\n$3Dmolpromise = null;\n  $3Dmolpromise = loadScriptAsync('https://cdn.jsdelivr.net/npm/3dmol@latest/build/3Dmol-min.min.js');\n}\n\nvar viewer_16692193574285905 = null;\nvar warn = document.getElementById(\"3dmolwarning_16692193574285905\");\nif(warn) {\n    warn.parentNode.removeChild(warn);\n}\n$3Dmolpromise.then(function() {\nviewer_16692193574285905 = $3Dmol.createViewer($(\"#3dmolviewer_16692193574285905\"),{backgroundColor:\"white\"});\nviewer_16692193574285905.zoomTo();\n\tviewer_16692193574285905.addModel(\"5\\n\\n O    1.338181708340   -0.344148062155   -0.119477105807\\n N    0.027181708340    0.013851937845    0.422522894193\\n O   -0.316818291660    1.320851937845   -0.137477105807\\n O   -0.931818291660   -0.944148062155   -0.127477105807\\n H   -1.798818291660   -0.709148062155    0.230522894193\\n\",\"xyz\");\n\tviewer_16692193574285905.setStyle({\"stick\": {}, \"sphere\": {\"scale\": 0.1}});\n\tviewer_16692193574285905.zoomTo();\nviewer_16692193574285905.render();\n});\n</script>", "text/html": ["<div id=\"3dmolviewer_16692193574285905\"  style=\"position: relative; width: 400px; height: 400px\">\n", "        <p id=\"3dmolwarning_16692193574285905\" style=\"background-color:#ffcccc;color:black\">You appear to be running in JupyterLab (or JavaScript failed to load for some other reason).  You need to install the 3dmol extension: <br>\n", "        <tt>jupyter labextension install jupyterlab_3dmol</tt></p>\n", "        </div>\n", "<script>\n", "\n", "var loadScriptAsync = function(uri){\n", "  return new Promise((resolve, reject) => {\n", "    var tag = document.createElement('script');\n", "    tag.src = uri;\n", "    tag.async = true;\n", "    tag.onload = () => {\n", "      resolve();\n", "    };\n", "  var firstScriptTag = document.getElementsByTagName('script')[0];\n", "  firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);\n", "});\n", "};\n", "\n", "if(typeof $3Dmolpromise === 'undefined') {\n", "$3Dmolpromise = null;\n", "  $3Dmolpromise = loadScriptAsync('https://cdn.jsdelivr.net/npm/3dmol@latest/build/3Dmol-min.min.js');\n", "}\n", "\n", "var viewer_16692193574285905 = null;\n", "var warn = document.getElementById(\"3dmolwarning_16692193574285905\");\n", "if(warn) {\n", "    warn.parentNode.removeChild(warn);\n", "}\n", "$3Dmolpromise.then(function() {\n", "viewer_16692193574285905 = $3Dmol.createViewer($(\"#3dmolviewer_16692193574285905\"),{backgroundColor:\"white\"});\n", "viewer_16692193574285905.zoomTo();\n", "\tviewer_16692193574285905.addModel(\"5\\n\\n O    1.338181708340   -0.344148062155   -0.119477105807\\n N    0.027181708340    0.013851937845    0.422522894193\\n O   -0.316818291660    1.320851937845   -0.137477105807\\n O   -0.931818291660   -0.944148062155   -0.127477105807\\n H   -1.798818291660   -0.709148062155    0.230522894193\\n\",\"xyz\");\n", "\tviewer_16692193574285905.setStyle({\"stick\": {}, \"sphere\": {\"scale\": 0.1}});\n", "\tviewer_16692193574285905.zoomTo();\n", "viewer_16692193574285905.render();\n", "});\n", "</script>"]}, "metadata": {}, "output_type": "display_data"}], "source": ["drawXYZ(hno3)"]}, {"cell_type": "code", "execution_count": 53, "id": "b2ae8ea7", "metadata": {}, "outputs": [], "source": ["psi4.set_options({'reference':'uhf','guess':'gwh'})"]}, {"cell_type": "code", "execution_count": 54, "id": "97637d62", "metadata": {}, "outputs": [], "source": ["hno3_hf =  hno3.clone()"]}, {"cell_type": "code", "execution_count": 55, "id": "2a36c687", "metadata": {}, "outputs": [{"name": "stdout", "output_type": "stream", "text": ["Optimizer: Optimization complete!\n"]}], "source": ["psi4.core.set_output_file(f'hno3_hf.log', False)\n", "E_hf, history_hf = psi4.optimize('scf/6-31+G*', molecule=hno3_hf, return_history=True)"]}, {"cell_type": "code", "execution_count": 45, "id": "0ae26eeb", "metadata": {}, "outputs": [], "source": ["hno3_mp2 = hno3_hf.clone()\n", "psi4.core.set_output_file(f'hno3_mp2.log', False)\n", "E_mp2, history_mp2 = None, None # run MP2 calculation here, need to use return_history=True to return the history as output (used later)"]}, {"cell_type": "code", "execution_count": null, "id": "6a4fe579", "metadata": {}, "outputs": [], "source": ["hno3_mp3 = hno3_mp2.clone()\n", "psi4.core.set_output_file(f'hno3_mp3.log', False)\n", "E_mp2, history_mp2 = None, None # run MP3 calculation here, need to use return_history=True to return the history as output (used later)"]}, {"cell_type": "code", "execution_count": 58, "id": "38a4eac6", "metadata": {}, "outputs": [], "source": ["#save last optimized coordinates in a dictionary\n", "opt_coord = {\n", "    'hf':np.array(history_hf['coordinates'][-1]),\n", "    'mp2':np.array(history_mp2['coordinates'][-1]),\n", "    'mp3':np.array(history_mp3['coordinates'][-1])\n", "}"]}, {"cell_type": "markdown", "id": "a61a2cf2", "metadata": {}, "source": ["You can calculate the distance between two points is the L2 norm (Euclidean distance) as\n", "\n", "```python\n", "a = np.array([0,0,0])\n", "b = np.array([1,1,1])\n", "np.linalg.norm(a-b)\n", "```\n", "\n", "this has been implemented in the `calculate_bond` function contained in `helpers.py` imported above. \n", "The calculation for the angle is also possible using numpy. You can use the following function:\n", "\n", "```python\n", "def calculate_angle(a,b,c):\n", "    ba = a - b\n", "    bc = c - b\n", "\n", "    cosine_angle = np.dot(ba, bc) / (np.linalg.norm(ba) * np.linalg.norm(bc))\n", "    angle = np.arccos(cosine_angle)\n", "    return np.degrees(angle)\n", "```"]}, {"cell_type": "code", "execution_count": 62, "id": "b509c015", "metadata": {}, "outputs": [{"data": {"application/3dmoljs_load.v0": "<div id=\"3dmolviewer_16692199545666447\"  style=\"position: relative; width: 400px; height: 400px\">\n        <p id=\"3dmolwarning_16692199545666447\" style=\"background-color:#ffcccc;color:black\">You appear to be running in JupyterLab (or JavaScript failed to load for some other reason).  You need to install the 3dmol extension: <br>\n        <tt>jupyter labextension install jupyterlab_3dmol</tt></p>\n        </div>\n<script>\n\nvar loadScriptAsync = function(uri){\n  return new Promise((resolve, reject) => {\n    var tag = document.createElement('script');\n    tag.src = uri;\n    tag.async = true;\n    tag.onload = () => {\n      resolve();\n    };\n  var firstScriptTag = document.getElementsByTagName('script')[0];\n  firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);\n});\n};\n\nif(typeof $3Dmolpromise === 'undefined') {\n$3Dmolpromise = null;\n  $3Dmolpromise = loadScriptAsync('https://cdn.jsdelivr.net/npm/3dmol@latest/build/3Dmol-min.min.js');\n}\n\nvar viewer_16692199545666447 = null;\nvar warn = document.getElementById(\"3dmolwarning_16692199545666447\");\nif(warn) {\n    warn.parentNode.removeChild(warn);\n}\n$3Dmolpromise.then(function() {\nviewer_16692199545666447 = $3Dmol.createViewer($(\"#3dmolviewer_16692199545666447\"),{backgroundColor:\"white\"});\nviewer_16692199545666447.zoomTo();\n\tviewer_16692199545666447.addModel(\"5\\n\\n O    1.338181708340   -0.344148062155   -0.119477105807\\n N    0.027181708340    0.013851937845    0.422522894193\\n O   -0.316818291660    1.320851937845   -0.137477105807\\n O   -0.931818291660   -0.944148062155   -0.127477105807\\n H   -1.798818291660   -0.709148062155    0.230522894193\\n\",\"xyz\");\n\tviewer_16692199545666447.addLabel(\"0\",{\"position\": {\"x\": 1.33818170834342, \"y\": -0.3441480621560101, \"z\": -0.11947710580699746}, \"inFront\": true});\n\tviewer_16692199545666447.addLabel(\"1\",{\"position\": {\"x\": 0.02718170834042214, \"y\": 0.013851937844808478, \"z\": 0.4225228941942418}, \"inFront\": true});\n\tviewer_16692199545666447.addLabel(\"2\",{\"position\": {\"x\": -0.3168182916603645, \"y\": 1.3208519378477972, \"z\": -0.13747710580703862}, \"inFront\": true});\n\tviewer_16692199545666447.addLabel(\"3\",{\"position\": {\"x\": -0.9318182916617707, \"y\": -0.944148062157382, \"z\": -0.12747710580701577}, \"inFront\": true});\n\tviewer_16692199545666447.addLabel(\"4\",{\"position\": {\"x\": -1.7988182916637532, \"y\": -0.7091480621568447, \"z\": 0.23052289419380287}, \"inFront\": true});\n\tviewer_16692199545666447.setStyle({\"stick\": {}, \"sphere\": {\"scale\": 0.1}});\n\tviewer_16692199545666447.setStyle({\"model\": 0},{\"stick\": {\"colorscheme\": \"yellowCarbon\"}, \"sphere\": {\"scale\": 0.1}});\n\tviewer_16692199545666447.zoomTo();\nviewer_16692199545666447.render();\n});\n</script>", "text/html": ["<div id=\"3dmolviewer_16692199545666447\"  style=\"position: relative; width: 400px; height: 400px\">\n", "        <p id=\"3dmolwarning_16692199545666447\" style=\"background-color:#ffcccc;color:black\">You appear to be running in JupyterLab (or JavaScript failed to load for some other reason).  You need to install the 3dmol extension: <br>\n", "        <tt>jupyter labextension install jupyterlab_3dmol</tt></p>\n", "        </div>\n", "<script>\n", "\n", "var loadScriptAsync = function(uri){\n", "  return new Promise((resolve, reject) => {\n", "    var tag = document.createElement('script');\n", "    tag.src = uri;\n", "    tag.async = true;\n", "    tag.onload = () => {\n", "      resolve();\n", "    };\n", "  var firstScriptTag = document.getElementsByTagName('script')[0];\n", "  firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);\n", "});\n", "};\n", "\n", "if(typeof $3Dmolpromise === 'undefined') {\n", "$3Dmolpromise = null;\n", "  $3Dmolpromise = loadScriptAsync('https://cdn.jsdelivr.net/npm/3dmol@latest/build/3Dmol-min.min.js');\n", "}\n", "\n", "var viewer_16692199545666447 = null;\n", "var warn = document.getElementById(\"3dmolwarning_16692199545666447\");\n", "if(warn) {\n", "    warn.parentNode.removeChild(warn);\n", "}\n", "$3Dmolpromise.then(function() {\n", "viewer_16692199545666447 = $3Dmol.createViewer($(\"#3dmolviewer_16692199545666447\"),{backgroundColor:\"white\"});\n", "viewer_16692199545666447.zoomTo();\n", "\tviewer_16692199545666447.addModel(\"5\\n\\n O    1.338181708340   -0.344148062155   -0.119477105807\\n N    0.027181708340    0.013851937845    0.422522894193\\n O   -0.316818291660    1.320851937845   -0.137477105807\\n O   -0.931818291660   -0.944148062155   -0.127477105807\\n H   -1.798818291660   -0.709148062155    0.230522894193\\n\",\"xyz\");\n", "\tviewer_16692199545666447.addLabel(\"0\",{\"position\": {\"x\": 1.33818170834342, \"y\": -0.3441480621560101, \"z\": -0.11947710580699746}, \"inFront\": true});\n", "\tviewer_16692199545666447.addLabel(\"1\",{\"position\": {\"x\": 0.02718170834042214, \"y\": 0.013851937844808478, \"z\": 0.4225228941942418}, \"inFront\": true});\n", "\tviewer_16692199545666447.addLabel(\"2\",{\"position\": {\"x\": -0.3168182916603645, \"y\": 1.3208519378477972, \"z\": -0.13747710580703862}, \"inFront\": true});\n", "\tviewer_16692199545666447.addLabel(\"3\",{\"position\": {\"x\": -0.9318182916617707, \"y\": -0.944148062157382, \"z\": -0.12747710580701577}, \"inFront\": true});\n", "\tviewer_16692199545666447.addLabel(\"4\",{\"position\": {\"x\": -1.7988182916637532, \"y\": -0.7091480621568447, \"z\": 0.23052289419380287}, \"inFront\": true});\n", "\tviewer_16692199545666447.setStyle({\"stick\": {}, \"sphere\": {\"scale\": 0.1}});\n", "\tviewer_16692199545666447.setStyle({\"model\": 0},{\"stick\": {\"colorscheme\": \"yellowCarbon\"}, \"sphere\": {\"scale\": 0.1}});\n", "\tviewer_16692199545666447.zoomTo();\n", "viewer_16692199545666447.render();\n", "});\n", "</script>"]}, "metadata": {}, "output_type": "display_data"}], "source": ["drawXYZ_labeled(hno3)"]}, {"cell_type": "code", "execution_count": 60, "id": "a647d0cb", "metadata": {}, "outputs": [], "source": ["# we use the opt_coord dictionary to calculate the properties using python \n", "# the central N has index 1, the oxygens have index 0,2,3\n", "# the units are in bohr, we convert to \u00c5\n", "results = {'hf':{},'mp2':{},'mp3':{},'exp':{'r(O0-N) \u00c5':1.198, 'r(O2-N) \u00c5':1.410, 'r(O3-N) \u00c5':1.213, '\u03c6(O-N-O) \u00b0':130.2}, }\n", "for method in ['hf', 'mp2', 'mp3']:\n", "    xyz = opt_coord[method] # store the geometry array optimized with the current method in a temp variable\n", "    results[method]['r(O0-N) \u00c5'] = calculate_bond(xyz[0], xyz[1]) * psi4.constants.bohr2angstroms\n", "    results[method]['r(O2-N) \u00c5'] = calculate_bond(xyz[2], xyz[1]) * psi4.constants.bohr2angstroms\n", "    results[method]['r(O3-N) \u00c5'] = calculate_bond(xyz[3], xyz[1]) * psi4.constants.bohr2angstroms\n", "    results[method]['\u03c6(O-N-O) \u00b0'] = calculate_angle(xyz[0], xyz[1], xyz[2])"]}, {"cell_type": "code", "execution_count": null, "id": "77e0fb97", "metadata": {}, "outputs": [], "source": ["pd.DataFrame.from_dict(results)"]}], "metadata": {"celltoolbar": "Tags", "kernelspec": {"display_name": "Computational Chemistry", "language": "python", "name": "compchem"}, "language_info": {"codemirror_mode": {"name": "ipython", "version": 3}, "file_extension": ".py", "mimetype": "text/x-python", "name": "python", "nbconvert_exporter": "python", "pygments_lexer": "ipython3", "version": "3.8.12"}, "unianalytics_cell_mapping": [["0b15b497", "0b15b497"], ["cb2ad91b-6b5b-4086-af86-dae18c128879", "cb2ad91b-6b5b-4086-af86-dae18c128879"], ["01a12e6f", "01a12e6f"], ["bf20f5df", "bf20f5df"], ["7f51ac46-0dba-42d7-9bed-65000e92f92b", "7f51ac46-0dba-42d7-9bed-65000e92f92b"], ["03cea67c", "03cea67c"], ["7c21bbc9", "7c21bbc9"], ["a587f1aa", "a587f1aa"], ["b2ae8ea7", "b2ae8ea7"], ["97637d62", "97637d62"], ["2a36c687", "2a36c687"], ["0ae26eeb", "0ae26eeb"], ["bcc34ee3", "bcc34ee3"], ["6a4fe579", "6a4fe579"], ["b2e9df4d", "b2e9df4d"], ["38a4eac6", "38a4eac6"], ["a61a2cf2", "a61a2cf2"], ["b509c015", "b509c015"], ["a647d0cb", "a647d0cb"], ["77e0fb97", "77e0fb97"], ["74700ec9", "74700ec9"]], "unianalytics_notebook_id": "0e40a13f-5d87-4863-b4da-e254b8d9ce24"}, "nbformat": 4, "nbformat_minor": 5}